#  src-config.mk - hhl - /usr/include/mk
#  Copyright 2020 Nathan Fisher <nfisher.sr@gmail.com>
#
include versions.mk
os_name        = Hitch Hiker Linux
os_abbrev      = hhl
os_version     = 2022q3
basedir        = /src
include $(basedir)/config.mk

arch          ?= $(shell uname -m)
bld_arch      ?= $(shell uname -m)

ifndef colormk
include hhl.colors.mk
endif

ifeq ($(filter armv6 armv7l aarch64, $(arch)), $(arch))
 tgt           = $(arch)-hitchhiker-linux-musleabihf
else
 tgt           = $(arch)-hitchhiker-linux-musl
endif
bld            = $(shell gcc -dumpmachine)

exec_prefix   ?= ${DESTDIR}${install_prefix}
bindir        ?= ${exec_prefix}/bin
sbindir       ?= ${exec_prefix}/sbin
libdir        ?= ${DESTDIR}${install_prefix}/lib
libexecdir    ?= ${DESTDIR}${install_prefix}/libexec
includedir    ?= ${DESTDIR}${install_prefix}/include
sharedir      ?= ${DESTDIR}${install_prefix}/share
docdir        ?= ${sharedir}/doc
infodir       ?= ${sharedir}/info
localedir     ?= ${sharedir}/locale
mandir        ?= ${sharedir}/man
man1dir        = ${mandir}/man1
man2dir        = ${mandir}/man2
man3dir        = ${mandir}/man3
man4dir        = ${mandir}/man4
man5dir        = ${mandir}/man5
man6dir        = ${mandir}/man6
man7dir        = ${mandir}/man7
man8dir        = ${mandir}/man8

distdir        = ${basedir}/distfiles
toolchain      = ${basedir}/toolchain
world          = ${basedir}/world
build          = ${basedir}/build/$(arch)
distfullname  ?= ${distname}-${${distname}_version}
objdirname    ?= obj
distext       ?= xz
distfile      ?= ${distfullname}.tar.${distext}
distlocal     ?= ${distdir}/${distfile}
wkgdir         = $(shell pwd)
srcdir        ?= $(global_srcdir)/$(distfullname)
objdir        ?= $(global_objdir)/$(distfullname)
tooldir        = $(basedir)/build/toolchain-$(arch)

export LC_ALL  = POSIX
export PATH    = $(tooldir)/bin:$(tooldir)/sbin:/usr/bin:/usr/bin/core_perl:/usr/sbin:/bin:/sbin

_dhcpcd       ?= 1
_wireless     ?= 1

use_configure ?= 1
no_objdir     ?= 0

ifeq ($(use_configure),1)
config_tgt    ?= $(objdir)/Makefile
config_deps   += $(srcdir)/.dirstamp
config_opts   ?= --prefix=$(install_prefix)
config_cmd    ?= cd $(objdir) && $(srcdir)/configure $(config_opts)
build_deps    ?= $(config_tgt)
else
build_deps    ?= $(config_deps)
build_deps    += $(srcdir)/.dirstamp
endif
build_cmd     ?= $(MAKE) $(makeflags) -C $(objdir)
install_deps  ?= $(objdir)/.built
install_cmd   ?= $(MAKE) $(makeflags) -C $(objdir) install
all_deps      ?= $(objdir)/.built

ifeq ($(no_objdir),1)
 objdir        = $(srcdir)
else
 pre_dirs     += $(objdir)
endif

ifdef patches
 config_deps  += $(patches) $(srcdir)/.patched
endif

stripdirs     += $(build)/lib
stripdirs     += $(build)/usr/lib
stripdirs     += $(build)/bin
stripdirs     += $(build)/sbin
stripdirs     += $(build)/bin
stripdirs     += $(build)/sbin
stripdirs     += $(build)/usr/bin
stripdirs     += $(build)/usr/sbin
stripdirs     += $(build)/usr/bin
stripdirs     += $(build)/usr/sbin

$(basedir)/config.mk:
	cp $(basedir)/config.mk.sample $(basedir)/config.mk

.DEFAULT_GOAL  = $(default_goal)
